#########################################################################
#
#    This file is part of LWX. 
#
#    LWX is the abbreviation of the Light WiMax simulation program which 
#    is a ns2 WiMAX network simulation module. LWX allows each user to 
#    apply his/her designed scheduling algorithm or other mechanism used 
#    in WiMAX network and generate the corresponding simulation results
#    for further analysis or other non-commercial purposes.
#
#    Copyright (C) 2010 Pandeli Kolomitro , School of Computing, Queen's University in CANADA 
#    E-Mail: pandeli@cs.queensu.ca
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##########################################################################
# Check input parameters
if {$argc != 0} {
	puts ""
	puts "Wrong Number of Arguments! No arguments in this topology"
	puts "Syntax: ../../software/ns-allinone-2.31-80216-nist-mpr/ns-2.31/./ns wimax_3RS_6SS_6SN.tcl nbMNs"
	puts ""
	exit (1)
}

# ======================================================================
# Default Script Options
# ======================================================================
set opt(chan)        Channel/WirelessChannel    ;#Channel Type
set opt(prop)        Propagation/TwoRayGround   ;# radio-propagation model
set opt(netif)       Phy/WirelessPhy            ;# network interface type
set opt(mac)         Mac/LWX                    ;# MAC type
set opt(ifq)         Queue/DropTail/PriQueue    ;# interface queue type
set opt(ll)          LL                         ;# link layer type
set opt(ant)         Antenna/OmniAntenna        ;# antenna model
set opt(ifqlen)      50                         ;# max packet in ifq
set mn               9         	        ;# max number of mobile nodes
set rs               2                          ;# max number of relay stations
set fsh_mn           10
set sdh_mn           10
set thdh_mn          10                           
set an               12                     ;# max number of all nodes (BS+RS+SS)                  
set opt(rp)          AODV                       ;# routing protocol. DSDV, DSR, AODV.
set opt(x)           2500                        ;# Topology size
set opt(y)           2500                        ;# Topology size
set opt(trace)       mf-test.tr       ;# trace file
set traffic_start 0.50
set traffic_stop  15.50
set diff 0.01
set diff2 0.02
set d 3                               ;# set number of DL nodes per serving station
set u 3                               ;# set numer of UL nodes per serving station
set h1 [expr 1 *$d]                   ;# first hop DL
set h2 [expr 2 *$d]
set h3 [expr 3 *$d] 
set h1u [expr 1 *$u]
set h2u [expr 2 *$u]
set h3u [expr 3 *$u] 

# ======================================================================
# Initialize Global Variables
# ======================================================================

set ns		[new Simulator]

# This command should be called before the universal trace command $ns trace-all 
$ns use-newtrace                                   

#set namtrace [open $opt(nam.tr) w]                    ;# for nam
#$ns namtrace-all-wireless $namtrace $opt(x) $opt(y)   ;# for nam

set tracefd     [open $opt(trace) w]
$ns trace-all $tracefd

proc finish { } {                               ;# new a procedure finish
     global ns namtrace tracefd opt             ;# set global variable 
     $ns flush-trace
     #close $namtrace                           ;# for nam
     close $tracefd                             ;# close file 
     #exec nam $opt(nam.tr) &                  ;# for nam
     exit 0
}


# set up topography object
set topo       [new Topography]
$topo load_flatgrid $opt(x) $opt(y)

# Create God
create-god $an         ;# number of mobile nodes plus base station

# Create channel
set chan_ [new $opt(chan)]



# Create node(0) and node(1)

# configure node, please note the change below.
$ns node-config -adhocRouting $opt(rp) \
		-llType $opt(ll) \
		-macType $opt(mac) \
		-ifqType $opt(ifq) \
		-ifqLen $opt(ifqlen) \
		-antType $opt(ant) \
		-propType $opt(prop) \
		-phyType $opt(netif) \
		-topoInstance $topo \
		-agentTrace ON \
		-routerTrace OFF \
		-macTrace OFF \
		-movementTrace OFF \
		-channel $chan_


# ======================================================================
# Set Bandwidth Allocation Algorithm Type
# 0: single frame
# 1: multi frame
# 2: Round Robin
# 3: Strict Priority
# 101: transparent RS
# ======================================================================

# AZ, RZ, and NH are utilized to configure the OFDMA frame 

Mac/LWX env BWA_Algorithm_Type 1 Nb_RS 1 AZ 1 NH 2  


# ======================================================================
# adding nodes
# ======================================================================


                       
set node_id(0)     0
set node(0)        [$ns node]
$node(0) random-motion 0
$ns initial_node_pos $node(0) 20
$node(0) set X_ 17.0
$node(0) set Y_ 17.0
$node(0) set Z_ 0.0
$node(0) nodeid $node_id(0)
$node(0) radius 3000
$node(0) color "0"
$node(0) shape "circle"
Mac/LWX env add_bs node_id $node_id(0)

for {set i 1} {$i < $rs} {incr i} {
   set node_id($i)      $i
   set node($i)        [$ns node]
   $node($i) random-motion 0
   $ns initial_node_pos $node($i) 20
   $node($i) set X_ [expr 36 + $i]
   $node($i) set Y_ 17.0
   $node($i) set Z_ 0.0
   $node($i) nodeid $node_id($i)
   Mac/LWX env add_rs node_id $node_id($i) bs_node_id $node_id(0)
}

# Creation of the mobile nodes 
for {set i $rs} {$i < [expr $rs + $h1]} {incr i} {
   set node_id($i)     [expr 100 + $i]
   set node($i)        [$ns node]
   $node($i) random-motion 0
   $ns initial_node_pos $node($i) 20
   $node($i) set X_ [expr rand()*10 + 27]
   $node($i) set Y_ [expr rand()*10 + 17.0]
   $node($i) set Z_ 0.0
  
   $node($i) color "blue"
   $node($i) shape "circle"

   $node($i) nodeid $node_id($i)
   Mac/LWX env add_ss node_id $node_id($i) bs_node_id $node_id(0)
} 

# Creation of the mobile nodes 
for {set i [expr $rs + $h1]} {$i < [expr $rs + $h2]} {incr i} {
   set node_id($i)     [expr 100 + $i]
   set node($i)        [$ns node]
   $node($i) random-motion 0
   $ns initial_node_pos $node($i) 20
   $node($i) set X_ [expr rand()*10 + 38]
   $node($i) set Y_ [expr rand()*10 + 17.0]
   $node($i) set Z_ 0.0
  
   $node($i) color "blue"
   $node($i) shape "circle"

   $node($i) nodeid $node_id($i)
   Mac/LWX env add_ss node_id $node_id($i) bs_node_id $node_id(0)
   Mac/LWX env attach_rs node_id $node_id($i) rs_node_id $node_id(1)
}
# Creation of the mobile nodes 
for {set i [expr $rs + $h2]} {$i < [expr $rs + $h3]} {incr i} {
   set node_id($i)     [expr 100 + $i]
   set node($i)        [$ns node]
   $node($i) random-motion 0
   $ns initial_node_pos $node($i) 20
   $node($i) set X_ [expr rand()*10 + 39]
   $node($i) set Y_ [expr rand()*10 + 17.0]
   $node($i) set Z_ 0.0
  
   $node($i) color "blue"
   $node($i) shape "circle"
   $node($i) nodeid $node_id($i)
   Mac/LWX env add_ss node_id $node_id($i) bs_node_id $node_id(0)
Mac/LWX env attach_rs node_id $node_id($i) rs_node_id $node_id(2)
}

# ======================================================================
# adding downlink flows
# ======================================================================

# BS DL
# Pandeli Creation of connections DL
for {set i $rs} {$i < [expr $rs + $h1]} {incr i} {

         set udp_($i) [new Agent/UDP]
         $ns attach-agent  $node(0)  $udp_($i)
         set null_($i)  [new Agent/Null]
         $ns attach-agent  $node($i)  $null_($i)

        
         set vbr_($i) [new Application/Traffic/Exponential]
         $vbr_($i) set packetSize_ 200           
         $vbr_($i) set rate_       80kb          
         $vbr_($i) set burst_time_ 147ms 
	 $vbr_($i) set idle_time_ 167ms 
         $vbr_($i) attach-agent $udp_($i)
         $ns connect $udp_($i) $null_($i)

 
      set flow_info     [$udp_($i) flow_info]
 
   # wimax
   # bs->ss
   # init env var
   set src_nid       $node_id(0)
   set from_nid      $node_id(0)
   set to_nid        $node_id($i)   
   #set fs_rs         $node_id
   # add flow
   # Mac/LWX env add_flow <type> $src_nid $from_nid $to_nid $flow_info
   # type: bs_to_ss, ss_to_bs, rs_to_bs, rs_to_ss

    Mac/LWX env add_flow bs_to_ss $src_nid $from_nid $to_nid $flow_info
    # set flow attribute
    # Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info <type> <value>
    # type: rate(byte), QoS_Class (1~5), Rmin(bps), Rmax(bps), Lmax(ms), Jitter(ms) 
  
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info rate       60
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info QoS_Class  2
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Rmin       10000
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Rmax       36000000
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Lmax       0.030
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Jitter     0.030

      $ns at [expr $traffic_start+$i*$diff] "$cbr_($i) start"
      $ns at [expr $traffic_stop+$i*$diff] "$cbr_($i) stop"
      puts [$udp_($i) flow_info]
}

# RS DL 2st-hop
for {set i [expr $rs + $h1]} {$i < [expr $rs + $h2]} {incr i} {

         set udp_($i) [new Agent/UDP]
         $ns attach-agent  $node(0)  $udp_($i)
         set null_($i)  [new Agent/Null]
         $ns attach-agent  $node($i)  $null_($i)

         set vbr_($i) [new Application/Traffic/Exponential]
         $vbr_($i) set packetSize_ 200          
         $vbr_($i) set rate_       80kb           
         $vbr_($i) set burst_time_ 147ms 
	 $vbr_($i) set idle_time_ 167ms 
         $vbr_($i) attach-agent $udp_($i)
         $ns connect $udp_($i) $null_($i)

  
      set flow_info     [$udp_($i) flow_info]
 
   # wimax
   # bs->ss
   # init env var
   set src_nid       $node_id(0)
   set from_nid      $node_id(0)
   set to_nid        $node_id($i)  
   #set fs_rs         $node_id
    
   # add flow
   # Mac/LWX env add_flow <type> $src_nid $from_nid $to_nid $flow_info
   # type: bs_to_ss, ss_to_bs, rs_to_bs, rs_to_ss

    Mac/LWX env add_flow bs_to_ss $src_nid $from_nid $to_nid $flow_info
    # set flow attribute
    # Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info <type> <value>
    # type: rate(byte), QoS_Class (1~5), Rmin(bps), Rmax(bps), Lmax(ms), Jitter(ms) 

    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info rate       0
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info QoS_Class  2
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Rmin       10000
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Rmax       36000000
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Lmax       0.030
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Jitter     0.030

    # adding this flow's relay link
   # Mac/LWX env set_relay <type> $src_nid $flow_info $rs_nid rate $relay_rate
   Mac/LWX env set_relay bs_to_rs $src_nid $flow_info $node_id(1) rate 27
   # adding relay link from rs to bs
   # relay link 1
   
    Mac/LWX env add_flow rs_to_ss_nontp $src_nid $node_id(1) $to_nid $flow_info
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info rate       60
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info QoS_Class  2
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Rmin       10000
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Rmax       36000000
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Lmax       0.030
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Jitter     0.030
   
      $ns at [expr $traffic_start+$i*$diff] "$vbr_($i) start"
      $ns at [expr $traffic_stop+$i*$diff] "$vbr_($i) stop"
      puts [$udp_($i) flow_info]
}

# MS DL 3nd-hop
for {set i [expr $rs + $h2]} {$i < [expr $rs + $h3]} {incr i} {
   
         set udp_($i) [new Agent/UDP]
         $ns attach-agent  $node(0)  $udp_($i)
         set null_($i)  [new Agent/Null]
         $ns attach-agent  $node($i)  $null_($i)

         set vbr_($i) [new Application/Traffic/Exponential]
         $vbr_($i) set packetSize_ 200           
         $vbr_($i) set rate_       80kb           
         $vbr_($i) set burst_time_ 147ms 
	 $vbr_($i) set idle_time_ 167ms 
         $vbr_($i) attach-agent $udp_($i)
         $ns connect $udp_($i) $null_($i)


     
      set flow_info     [$udp_($i) flow_info]


 
   # wimax
   # bs->ss
   # init env var
   set src_nid       $node_id(0)
   set from_nid      $node_id(0)
   set to_nid        $node_id($i)    ;# $node_id($i)
     
   # add flow
   # Mac/LWX env add_flow <type> $src_nid $from_nid $to_nid $flow_info
   # type: bs_to_ss, ss_to_bs, rs_to_bs, rs_to_ss

    Mac/LWX env add_flow bs_to_ss $src_nid $from_nid $to_nid $flow_info
    # set flow attribute
    # Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info <type> <value>
    # type: rate(byte), QoS_Class (1~5), Rmin(bps), Rmax(bps), Lmax(ms), Jitter(ms) 

    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info rate       0
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info QoS_Class  2
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Rmin       10000
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Rmax       36000000
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Lmax       0.030
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Jitter     0.030

    # adding this flow's relay link
   # Mac/LWX env set_relay <type> $src_nid $flow_info $rs_nid rate $relay_rate
   Mac/LWX env set_relay bs_to_rs $src_nid $flow_info $node_id(1) rate 27
   # adding relay link from rs to bs
   # relay link 1
   
    Mac/LWX env add_flow rs_to_ss_nontp $src_nid $node_id(1) $to_nid $flow_info
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info rate       0
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info QoS_Class  2
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Rmin       10000
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Rmax       36000000
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Lmax       0.030
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Jitter     0.030

    # adding this flow's relay link
   # Mac/LWX env set_relay <type> $src_nid $flow_info $rs_nid rate $relay_rate
   Mac/LWX env set_relay rs_to_rs_nontp_dl_odd $node_id(1) $flow_info $node_id(2) rate 27
   # adding relay link from rs to bs
   # relay link 1
   
    Mac/LWX env add_flow rs_to_ss_nontp $src_nid $node_id(2) $to_nid $flow_info
    Mac/LWX env set_flow $src_nid $node_id(2) $to_nid $flow_info rate       60
    Mac/LWX env set_flow $src_nid $node_id(2) $to_nid $flow_info QoS_Class  2
    Mac/LWX env set_flow $src_nid $node_id(2) $to_nid $flow_info Rmin       10000
    Mac/LWX env set_flow $src_nid $node_id(2) $to_nid $flow_info Rmax       36000000
    Mac/LWX env set_flow $src_nid $node_id(2) $to_nid $flow_info Lmax       0.030
    Mac/LWX env set_flow $src_nid $node_id(2) $to_nid $flow_info Jitter     0.030

      $ns at [expr $traffic_start+$i*$diff] "$vbr_($i) start"
      $ns at [expr  $traffic_stop+$i*$diff] "$vbr_($i) stop"
      puts [$udp_($i) flow_info]
}

#================================
#   UL Flow
#================================

# BS MN UL
for {set i $rs} {$i < [expr $rs + $h1u]} {incr i} {
     
         set udp_($i) [new Agent/UDP]
         $ns attach-agent  $node($i)  $udp_($i)
         set null_($i)  [new Agent/Null]
         $ns attach-agent  $node(0)  $null_($i)

         set vbr_($i) [new Application/Traffic/Exponential]
         $vbr_($i) set packetSize_ 200            
         $vbr_($i) set rate_       80kb           
         $vbr_($i) set burst_time_ 147ms 
	 $vbr_($i) set idle_time_ 167ms 
         $vbr_($i) attach-agent $udp_($i)
         $ns connect $udp_($i) $null_($i)

      set flow_info     [$udp_($i) flow_info]
 
   # wimax
   # bs->ss
   # init env var
   set src_nid       $node_id($i)
   set from_nid      $node_id($i)
   set to_nid        $node_id(0)    
   #set fs_rs         $node_id
   # add flow
   # Mac/LWX env add_flow <type> $src_nid $from_nid $to_nid $flow_info
   # type: bs_to_ss, ss_to_bs, rs_to_bs, rs_to_ss

    Mac/LWX env add_flow ss_to_bs $src_nid $from_nid $to_nid $flow_info
    # set flow attribute
    # Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info <type> <value>
    # type: rate(byte), QoS_Class (1~5), Rmin(bps), Rmax(bps), Lmax(ms), Jitter(ms) 

    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info rate       60
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info QoS_Class  2
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Rmin       10000
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Rmax       36000000
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Lmax       0.030
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Jitter     0.030

      $ns at [expr $traffic_start+$i*$diff2] "$vbr_($i) start"
      $ns at [expr $traffic_stop+$i*$diff2] "$vbr_($i) stop"
      puts [$udp_($i) flow_info]

}

# RS UL 2nd-hop
for {set i [expr $rs + $h1u]} {$i < [expr $rs + $h2u]} {incr i} {
     
         set udp_($i) [new Agent/UDP]
         $ns attach-agent  $node($i)  $udp_($i)
         set null_($i)  [new Agent/Null]
         $ns attach-agent  $node(0)  $null_($i)

         set vbr_($i) [new Application/Traffic/Exponential]
         $vbr_($i) set packetSize_ 200           
         $vbr_($i) set rate_       80kb           
         $vbr_($i) set burst_time_ 147ms 
	 $vbr_($i) set idle_time_ 167ms 
         $vbr_($i) attach-agent $udp_($i)
         $ns connect $udp_($i) $null_($i)

      set flow_info     [$udp_($i) flow_info]
 
   # wimax
   # bs->ss
   # init env var
   set src_nid       $node_id($i)
   set from_nid      $node_id($i)
   set to_nid        $node_id(0)    

  
   # add flow
   # Mac/LWX env add_flow <type> $src_nid $from_nid $to_nid $flow_info
   # type: bs_to_ss, ss_to_bs, rs_to_bs, rs_to_ss

    Mac/LWX env add_flow ss_to_bs $src_nid $from_nid $to_nid $flow_info
    # set flow attribute
    # Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info <type> <value>
    # type: rate(byte), QoS_Class (1~5), Rmin(bps), Rmax(bps), Lmax(ms), Jitter(ms) 

    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info rate       0
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info QoS_Class  2
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Rmin       10000
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Rmax       36000000
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Lmax       0.030
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Jitter     0.030

    Mac/LWX env set_relay ss_to_rs_nontp $src_nid $flow_info $node_id(1) rate 27
   
    Mac/LWX env add_flow rs_to_bs $src_nid $node_id(1) $to_nid $flow_info
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info rate       60 
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info QoS_Class  2
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Rmin       10000
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Rmax       36000000
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Lmax       0.030
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Jitter     0.030
  
      $ns at [expr $traffic_start+$i*$diff2] "$vbr_($i) start"
      $ns at [expr $traffic_stop+$i*$diff2] "$vbr_($i) stop"
      puts [$udp_($i) flow_info]
}

# MS UL 3d-hop RS
for {set i [expr $rs + $h2u]} {$i < [expr $rs + $h3u]} {incr i} {
    
         set udp_($i) [new Agent/UDP]
         $ns attach-agent  $node($i)  $udp_($i)
         set null_($i)  [new Agent/Null]
         $ns attach-agent  $node(0)  $null_($i)

         set vbr_($i) [new Application/Traffic/Exponential]
         $vbr_($i) set packetSize_ 200            
         $vbr_($i) set rate_       80kb           
         $vbr_($i) set burst_time_ 147ms 
	 $vbr_($i) set idle_time_ 167ms 
         $vbr_($i) attach-agent $udp_($i)
         $ns connect $udp_($i) $null_($i)

      set flow_info     [$udp_($i) flow_info]


 
   # wimax
   # bs->ss
   # init env var
   set src_nid       $node_id($i)
   set from_nid      $node_id($i)
   set to_nid        $node_id(0)   
   #set fs_rs         $node_id
 
   # add flow
   # Mac/LWX env add_flow <type> $src_nid $from_nid $to_nid $flow_info
   # type: bs_to_ss, ss_to_bs, rs_to_bs, rs_to_ss

    Mac/LWX env add_flow ss_to_bs $src_nid $from_nid $to_nid $flow_info
    # set flow attribute
    # Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info <type> <value>
    # type: rate(byte), QoS_Class (1~5), Rmin(bps), Rmax(bps), Lmax(ms), Jitter(ms) 

    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info rate       0
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info QoS_Class  2
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Rmin       10000
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Rmax       36000000
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Lmax       0.030
    Mac/LWX env set_flow $src_nid $from_nid $to_nid $flow_info Jitter     0.030

    # adding this flow's relay link
   # Mac/LWX env set_relay <type> $src_nid $flow_info $rs_nid rate $relay_rate
   Mac/LWX env set_relay ss_to_rs_nontp $src_nid $flow_info $node_id(2) rate 27
   # adding relay link from rs to bs
   # relay link 1
   
    Mac/LWX env add_flow rs_to_bs $src_nid $node_id(2) $to_nid $flow_info
    Mac/LWX env set_flow $src_nid $node_id(2) $to_nid $flow_info rate       0
    Mac/LWX env set_flow $src_nid $node_id(2) $to_nid $flow_info QoS_Class  2
    Mac/LWX env set_flow $src_nid $node_id(2) $to_nid $flow_info Rmin       10000
    Mac/LWX env set_flow $src_nid $node_id(2) $to_nid $flow_info Rmax       36000000
    Mac/LWX env set_flow $src_nid $node_id(2) $to_nid $flow_info Lmax       0.030
    Mac/LWX env set_flow $src_nid $node_id(2) $to_nid $flow_info Jitter     0.030

    # adding this flow's relay link
   # Mac/LWX env set_relay <type> $src_nid $flow_info $rs_nid rate $relay_rate
   Mac/LWX env set_relay rs_to_rs_nontp_ul_odd $node_id(2) $flow_info $node_id(1) rate 27
   # adding relay link from rs to bs
   # relay link 1
   
    Mac/LWX env add_flow rs_to_bs $src_nid $node_id(1) $to_nid $flow_info
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info rate       60
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info QoS_Class  2
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Rmin       10000
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Rmax       36000000
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Lmax       0.030
    Mac/LWX env set_flow $src_nid $node_id(1) $to_nid $flow_info Jitter     0.030
  
      $ns at [expr $traffic_start+$i*$diff2] "$vbr_($i) start"
      $ns at [expr  $traffic_stop+$i*$diff2] "$vbr_($i) stop"
      puts [$udp_($i) flow_info]

}

# Mac/LWX eventtrace ; 
#Mac/LWX env get_flow_info
#
# Tell nodes when the simulation ends
#
#for {set i 0} {$i < $an } {incr i} {
#    $ns at 1990.0 "$node($i) reset";
#}
$ns at 1000.0 "finish"
$ns at 1001.01 "puts \"NS EXITING...\" ; $ns halt"

puts "Starting Simulation..."
$ns run
